@@ -1,3 +1,12 @@
+/*
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ *
+ * This file is part of zmqpp.
+ * Copyright (c) 2011-2015 Contributors as noted in the AUTHORS file.
+ */
+
 /*
  *  Created on: 9 Aug 2011
  *      Author: Ben Gray (@benjamg)
@@ -16,17 +25,21 @@
 namespace zmqpp
 {
 
-const int socket::normal;
-const int socket::dont_wait;
-const int socket::send_more;
+const int socket::normal = 0;
+#if (ZMQ_VERSION_MAJOR == 2)
+const int socket::dont_wait = ZMQ_NOBLOCK;
+#else
+const int socket::dont_wait = ZMQ_DONTWAIT;
+#endif
+const int socket::send_more = ZMQ_SNDMORE;
 #ifdef ZMQ_EXPERIMENTAL_LABELS
-const int socket::send_label;
+const int socket::send_label = ZMQ_SNDLABEL;
 #endif
 
 const int max_socket_option_buffer_size = 256;
 const int max_stream_buffer_size = 4096;
 
-socket::socket(const context& context, socket_type const& type)
+socket::socket(const context& context, socket_type const type)
 	: _socket(nullptr)
 	, _type(type)
 	, _recv_buffer()
@@ -68,6 +81,7 @@ void socket::bind(endpoint_t const& endpoint)
 	}
 }
 
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 2))
 void socket::unbind(endpoint_t const& endpoint)
 {
 	int result = zmq_unbind(_socket, endpoint.c_str());
@@ -77,6 +91,7 @@ void socket::unbind(endpoint_t const& endpoint)
 		throw zmq_internal_exception();
 	}
 }
+#endif
 
 void socket::connect(endpoint_t const& endpoint)
 {
@@ -88,6 +103,7 @@ void socket::connect(endpoint_t const& endpoint)
 	}
 }
 
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 2))
 void socket::disconnect(endpoint_t const& endpoint)
 {
 	int result = zmq_disconnect(_socket, endpoint.c_str());
@@ -97,6 +113,7 @@ void socket::disconnect(endpoint_t const& endpoint)
 		throw zmq_internal_exception();
 	}
 }
+#endif
 
 void socket::close()
 {
@@ -110,7 +127,25 @@ void socket::close()
 	_socket = nullptr;
 }
 
-bool socket::send(message& message, bool const& dont_block /* = false */)
+bool socket::send(zmqpp::signal sig, bool dont_block/* = false */)
+{
+    message msg(sig);
+    return send(msg, dont_block);
+}
+
+  bool socket::receive(zmqpp::signal &sig, bool dont_block /* = false */)
+{
+    message msg;
+    bool ret = receive(msg, dont_block);
+    if (ret)
+    {
+        assert(msg.is_signal());
+        msg.get(sig, 0);
+    }
+    return ret;
+}
+
+bool socket::send(message& message, bool const dont_block /* = false */)
 {
 	size_t parts = message.parts();
 	if (parts == 0)
@@ -118,15 +153,16 @@ bool socket::send(message& message, bool const& dont_block /* = false */)
 		throw std::invalid_argument("sending requires messages have at least one part");
 	}
 
+	bool dont_wait = dont_block;
 	for(size_t i = 0; i < parts; ++i)
 	{
 		int flag = socket::normal;
-		if(dont_block) { flag |= socket::dont_wait; }
+		if(dont_wait) { flag |= socket::dont_wait; }
 		if(i < (parts - 1)) { flag |= socket::send_more; }
 
 #if (ZMQ_VERSION_MAJOR == 2)
 		int result = zmq_send( _socket, &message.raw_msg(i), flag );
-#elif (ZMQ_VERSION_MAJOR < 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR < 2))
+#elif (ZMQ_VERSION_MAJOR < 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR < 2))
 		int result = zmq_sendmsg( _socket, &message.raw_msg(i), flag );
 #else
 		int result = zmq_msg_send( &message.raw_msg(i), _socket, flag );
@@ -141,6 +177,20 @@ bool socket::send(message& message, bool const& dont_block /* = false */)
 				return false;
 			}
 
+			if(EINTR == zmq_errno())
+			{
+				if (0 == message.parts())
+				{
+					return false;
+				}
+
+				// If we have an interrupt but it's not on the first part then we
+				// know we can safely send out the rest of the message as we can
+				// enforce that it won't wait on a blocking action
+				dont_wait = true;
+				continue;
+			}
+
 			// sanity checking
 			assert(EAGAIN != zmq_errno());
 
@@ -156,11 +206,13 @@ bool socket::send(message& message, bool const& dont_block /* = false */)
 	return true;
 }
 
-bool socket::receive(message& message, bool const& dont_block /* = false */)
+bool socket::receive(message& message, bool const dont_block /* = false */)
 {
 	if (message.parts() > 0)
 	{
-		throw exception("receiving can only be done to empty messages");
+		// swap and discard old message
+		zmqpp::message local;
+		std::swap(local, message);
 	}
 
 	int flags = (dont_block) ? socket::dont_wait : socket::normal;
@@ -170,7 +222,7 @@ bool socket::receive(message& message, bool const& dont_block /* = false */)
 	{
 #if (ZMQ_VERSION_MAJOR == 2)
 		int result = zmq_recv( _socket, &_recv_buffer, flags );
-#elif (ZMQ_VERSION_MAJOR < 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR < 2))
+#elif (ZMQ_VERSION_MAJOR < 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR < 2))
 		int result = zmq_recvmsg( _socket, &_recv_buffer, flags );
 #else
 		int result = zmq_msg_recv( &_recv_buffer, _socket, flags );
@@ -183,7 +235,20 @@ bool socket::receive(message& message, bool const& dont_block /* = false */)
 				return false;
 			}
 
-			assert(EAGAIN == zmq_errno());
+			if(EINTR == zmq_errno())
+			{
+				if (0 == message.parts())
+				{
+					return false;
+				}
+
+				// If we have an interrupt but it's not on the first part then we
+				// know we can safely pull out the rest of the message as it will
+				// not be blocking
+				continue;
+			}
+
+			assert(EAGAIN != zmq_errno());
 
 			throw zmq_internal_exception();
 		}
@@ -198,16 +263,16 @@ bool socket::receive(message& message, bool const& dont_block /* = false */)
 }
 
 
-bool socket::send(std::string const& string, int const& flags /* = NORMAL */)
+bool socket::send(std::string const& string, int const flags /* = NORMAL */)
 {
 	return send_raw(string.data(), string.size(), flags);
 }
 
-bool socket::receive(std::string& string, int const& flags /* = NORMAL */)
+bool socket::receive(std::string& string, int const flags /* = NORMAL */)
 {
 #if (ZMQ_VERSION_MAJOR == 2)
 		int result = zmq_recv( _socket, &_recv_buffer, flags );
-#elif (ZMQ_VERSION_MAJOR < 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR < 2))
+#elif (ZMQ_VERSION_MAJOR < 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR < 2))
 		int result = zmq_recvmsg( _socket, &_recv_buffer, flags );
 #else
 		int result = zmq_msg_recv( &_recv_buffer, _socket, flags );
@@ -221,7 +286,7 @@ bool socket::receive(std::string& string, int const& flags /* = NORMAL */)
 		return true;
 	}
 
-	if (EAGAIN == zmq_errno())
+	if (EAGAIN == zmq_errno() || EINTR == zmq_errno())
 	{
 		return false;
 	}
@@ -230,7 +295,7 @@ bool socket::receive(std::string& string, int const& flags /* = NORMAL */)
 }
 
 
-bool socket::send_raw(char const* buffer, int const& length, int const& flags /* = NORMAL */)
+bool socket::send_raw(char const* buffer, size_t const length, int const flags /* = NORMAL */)
 {
 #if (ZMQ_VERSION_MAJOR == 2)
     zmq_msg_t msg;
@@ -255,7 +320,7 @@ bool socket::send_raw(char const* buffer, int const& length, int const& flags /*
     zmq_msg_close(&msg);
 #endif
 
-	if (EAGAIN == zmq_errno())
+	if (EAGAIN == zmq_errno() || EINTR == zmq_errno())
 	{
 		return false;
 	}
@@ -263,11 +328,11 @@ bool socket::send_raw(char const* buffer, int const& length, int const& flags /*
 	throw zmq_internal_exception();
 }
 
-bool socket::receive_raw(char* buffer, int& length, int const& flags /* = NORMAL */)
+bool socket::receive_raw(char* buffer, size_t& length, int const flags /* = NORMAL */)
 {
 #if (ZMQ_VERSION_MAJOR == 2)
 		int result = zmq_recv( _socket, &_recv_buffer, flags );
-#elif (ZMQ_VERSION_MAJOR < 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR < 2))
+#elif (ZMQ_VERSION_MAJOR < 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR < 2))
 		int result = zmq_recvmsg( _socket, &_recv_buffer, flags );
 #else
 		int result = zmq_msg_recv( &_recv_buffer, _socket, flags );
@@ -281,7 +346,7 @@ bool socket::receive_raw(char* buffer, int& length, int const& flags /* = NORMAL
 		return true;
 	}
 
-	if (EAGAIN == zmq_errno())
+	if (EAGAIN == zmq_errno() || EINTR == zmq_errno())
 	{
 		return false;
 	}
@@ -308,7 +373,7 @@ bool socket::has_more_parts() const
 
 
 // Set socket options for different types of option
-void socket::set(socket_option const& option, int const& value)
+void socket::set(socket_option const option, int const value)
 {
 	switch(option)
 	{
@@ -336,16 +401,30 @@ void socket::set(socket_option const& option, int const& value)
 		break;
 
 	// Boolean
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 1))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 1))
 	case socket_option::ipv4_only:
 #endif
 #if (ZMQ_VERSION_MAJOR == 2)
 	case socket_option::multicast_loopback:
 #endif
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR == 3 && ZMQ_VERSION_MINOR == 2)
 	case socket_option::delay_attach_on_connect:
+#else
+	case socket_option::immediate:
+#endif
 	case socket_option::router_mandatory:
 	case socket_option::xpub_verbose:
+#endif
+#if (ZMQ_VERSION_MAJOR >= 4)
+	case socket_option::ipv6:
+	case socket_option::plain_server:
+	case socket_option::conflate:
+	case socket_option::curve_server:
+	case socket_option::probe_router:
+	case socket_option::request_correlate:
+	case socket_option::request_relaxed:
+	case socket_option::router_raw:
 #endif
 		if (value == 0) { set(option, false); }
 		else if (value == 1) { set(option, true); }
@@ -353,7 +432,7 @@ void socket::set(socket_option const& option, int const& value)
 		break;
 
 	// Default or Boolean
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 2))
 	case socket_option::tcp_keepalive:
 		if (value < -1 || value > 1) { throw exception("attempting to set a default or boolean option with a non -1, 0 or 1 integer"); }
 		if (0 != zmq_setsockopt(_socket, static_cast<int>(option), &value, sizeof(value)))
@@ -364,6 +443,7 @@ void socket::set(socket_option const& option, int const& value)
 #endif
 
 	// Integers that require +ve numbers
+	case socket_option::backlog:
 #if (ZMQ_VERSION_MAJOR == 2)
 	case socket_option::reconnect_interval_max:
 #else
@@ -376,14 +456,13 @@ void socket::set(socket_option const& option, int const& value)
 	case socket_option::multicast_hops:
 	case socket_option::rate:
 #endif
-	case socket_option::backlog:
 		if (value < 0) { throw exception("attempting to set a positive only integer option with a negative integer"); }
 		// Integers
 	case socket_option::reconnect_interval:
 	case socket_option::linger:
 	case socket_option::receive_timeout:
 	case socket_option::send_timeout:
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 2))
 	case socket_option::tcp_keepalive_idle:
 	case socket_option::tcp_keepalive_count:
 	case socket_option::tcp_keepalive_interval:
@@ -398,29 +477,49 @@ void socket::set(socket_option const& option, int const& value)
 	}
 }
 
-void socket::set(socket_option const& option, bool const& value)
+void socket::set(socket_option const option, bool const value)
 {
 	switch(option)
 	{
 #if (ZMQ_VERSION_MAJOR == 2)
 	case socket_option::multicast_loopback:
 #endif
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 1))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 1))
 	case socket_option::ipv4_only:
 #endif
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR == 3 && ZMQ_VERSION_MINOR == 2)
 	case socket_option::delay_attach_on_connect:
+#else
+	case socket_option::immediate:
+#endif
 	case socket_option::router_mandatory:
 	case socket_option::xpub_verbose:
 #endif
-		zmq_setsockopt(_socket, static_cast<int>(option), &value, sizeof(value));
+#if (ZMQ_VERSION_MAJOR >= 4)
+	case socket_option::ipv6:
+	case socket_option::plain_server:
+	case socket_option::conflate:
+	case socket_option::curve_server:
+	case socket_option::probe_router:
+	case socket_option::request_correlate:
+	case socket_option::request_relaxed:
+	case socket_option::router_raw:
+#endif
+	{
+		int ivalue = value ? 1 : 0;
+		if (0 != zmq_setsockopt(_socket, static_cast<int>(option), &ivalue, sizeof(ivalue)))
+		{
+			throw zmq_internal_exception();
+		}
 		break;
+	}
 	default:
 		throw exception("attempting to set a non boolean option with a boolean value");
 	}
 }
 
-void socket::set(socket_option const& option, uint64_t const& value)
+void socket::set(socket_option const option, uint64_t const value)
 {
 	switch(option)
 	{
@@ -441,7 +540,7 @@ void socket::set(socket_option const& option, uint64_t const& value)
 	}
 }
 
-void socket::set(socket_option const& option, int64_t const& value)
+void socket::set(socket_option const option, int64_t const value)
 {
 	switch(option)
 	{
@@ -465,15 +564,23 @@ void socket::set(socket_option const& option, int64_t const& value)
 	}
 }
 
-void socket::set(socket_option const& option, char const* value, size_t const length)
+void socket::set(socket_option const option, char const* value, size_t const length)
 {
 	switch(option)
 	{
 	case socket_option::identity:
 	case socket_option::subscribe:
 	case socket_option::unsubscribe:
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 2))
 	case socket_option::tcp_accept_filter:
+#endif
+#if (ZMQ_VERSION_MAJOR >= 4)
+	case socket_option::plain_password:
+	case socket_option::plain_username:
+	case socket_option::zap_domain:
+	case socket_option::curve_public_key:
+	case socket_option::curve_secret_key:
+	case socket_option::curve_server_key:
 #endif
 		if (0 != zmq_setsockopt(_socket, static_cast<int>(option), value, length))
 		{
@@ -486,7 +593,7 @@ void socket::set(socket_option const& option, char const* value, size_t const le
 }
 
 // Get socket options, multiple versions for easy of use
-void socket::get(socket_option const& option, int& value) const
+void socket::get(socket_option const option, int& value) const
 {
 	size_t value_size = sizeof(int);
 
@@ -517,11 +624,15 @@ void socket::get(socket_option const& option, int& value) const
 	case socket_option::receive_high_water_mark:
 	case socket_option::multicast_hops:
 #endif
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 1))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 1))
 	case socket_option::ipv4_only:
 #endif
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR == 3 && ZMQ_VERSION_MINOR == 2)
 	case socket_option::delay_attach_on_connect:
+#else
+	case socket_option::immediate:
+#endif
 	case socket_option::tcp_keepalive:
 	case socket_option::tcp_keepalive_idle:
 	case socket_option::tcp_keepalive_count:
@@ -529,6 +640,11 @@ void socket::get(socket_option const& option, int& value) const
 #endif
 #ifdef ZMQ_EXPERIMENTAL_LABELS
 	case socket_option::receive_label:
+#endif
+#if (ZMQ_VERSION_MAJOR >= 4)
+	case socket_option::ipv6:
+	case socket_option::plain_server:
+	case socket_option::mechanism:
 #endif
 		if (0 != zmq_getsockopt(_socket, static_cast<int>(option), &value, &value_size))
 		{
@@ -543,7 +659,7 @@ void socket::get(socket_option const& option, int& value) const
 	}
 }
 
-void socket::get(socket_option const& option, bool& value) const
+void socket::get(socket_option const option, bool& value) const
 {
 #if (ZMQ_VERSION_MAJOR == 2)
 	int64_t int_value = 0;
@@ -559,14 +675,22 @@ void socket::get(socket_option const& option, bool& value) const
 #if (ZMQ_VERSION_MAJOR == 2)
 	case socket_option::multicast_loopback:
 #endif
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 1))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 1))
 	case socket_option::ipv4_only:
 #endif
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR == 3 && ZMQ_VERSION_MINOR == 2)
 	case socket_option::delay_attach_on_connect:
+#else
+	case socket_option::immediate:
+#endif
 #endif
 #ifdef ZMQ_EXPERIMENTAL_LABELS
 	case socket_option::receive_label:
+#endif
+#if (ZMQ_VERSION_MAJOR >= 4)
+	case socket_option::ipv6:
+	case socket_option::plain_server:
 #endif
 		if (0 != zmq_getsockopt(_socket, static_cast<int>(option), &int_value, &value_size))
 		{
@@ -580,7 +704,7 @@ void socket::get(socket_option const& option, bool& value) const
 	}
 }
 
-void socket::get(socket_option const& option, uint64_t& value) const
+void socket::get(socket_option const option, uint64_t& value) const
 {
 	size_t value_size = sizeof(uint64_t);
 
@@ -602,7 +726,7 @@ void socket::get(socket_option const& option, uint64_t& value) const
 	}
 }
 
-void socket::get(socket_option const& option, int64_t& value) const
+void socket::get(socket_option const option, int64_t& value) const
 {
 	size_t value_size = sizeof(int64_t);
 
@@ -628,7 +752,7 @@ void socket::get(socket_option const& option, int64_t& value) const
 	}
 }
 
-void socket::get(socket_option const& option, std::string& value) const
+void socket::get(socket_option const option, std::string& value) const
 {
 	static std::array<char, max_socket_option_buffer_size> buffer;
 	size_t size = max_socket_option_buffer_size;
@@ -636,8 +760,16 @@ void socket::get(socket_option const& option, std::string& value) const
 	switch(option)
 	{
 	case socket_option::identity:
-#if (ZMQ_VERSION_MAJOR > 3) or ((ZMQ_VERSION_MAJOR == 3) and (ZMQ_VERSION_MINOR >= 2))
+#if (ZMQ_VERSION_MAJOR > 3) || ((ZMQ_VERSION_MAJOR == 3) && (ZMQ_VERSION_MINOR >= 2))
 	case socket_option::last_endpoint:
+#endif
+#if (ZMQ_VERSION_MAJOR >= 4)
+	case socket_option::plain_password:
+	case socket_option::plain_username:
+	case socket_option::zap_domain:
+	case socket_option::curve_public_key:
+	case socket_option::curve_secret_key:
+	case socket_option::curve_server_key:
 #endif
 		if(0 != zmq_getsockopt(_socket, static_cast<int>(option), buffer.data(), &size))
 		{
@@ -651,7 +783,7 @@ void socket::get(socket_option const& option, std::string& value) const
 	}
 }
 
-socket::socket(socket&& source) noexcept
+socket::socket(socket&& source) NOEXCEPT
 	: _socket(source._socket)
 	, _type(source._type)
 	, _recv_buffer()
@@ -665,7 +797,7 @@ socket::socket(socket&& source) noexcept
 	source._socket = nullptr;
 }
 
-socket& socket::operator=(socket&& source) noexcept
+socket& socket::operator=(socket&& source) NOEXCEPT
 {
 	std::swap(_socket, source._socket);
 
@@ -691,7 +823,7 @@ socket::operator void*() const
 	return _socket;
 }
 
-void socket::track_message(message const& /* message */, uint32_t const& parts, bool& should_delete)
+void socket::track_message(message const& /* message */, uint32_t const parts, bool& should_delete)
 {
 	if (parts == 0)
 	{
@@ -699,4 +831,32 @@ void socket::track_message(message const& /* message */, uint32_t const& parts,
 	}
 }
 
+#if (ZMQ_VERSION_MAJOR >= 4)
+void socket::monitor(endpoint_t const monitor_endpoint, int events_required)
+{
+	int result = zmq_socket_monitor( _socket, monitor_endpoint.c_str(), events_required );
+
+	if (0 != result)
+	{
+		throw zmq_internal_exception();
+	}
+}
+#endif
+
+signal socket::wait()
+{
+    for (;;)
+    {
+        message msg;
+        while(!receive(msg));
+
+        if (msg.is_signal())
+        {
+            signal sig;
+            msg.get(sig, 0);
+            return sig;
+        }
+    }
+}
+
 }
